//-----------------------------------------------------------------------------
// x10 rf generator avec dsPIC 30F3010 et MO-SAWR-A (315 MHz, ASK)
//
// L. BAGHLI 17/12/2007 
//-----------------------------------------------------------------------------
#include "p30F3010.h"

//Configuration bits
/// pas de Quartz !
_FOSC(CSW_FSCM_OFF & FRC_PLL8);	// 7.37Mhz *8 = 58.96 /4 = 14.74 MIPS maxi pour ce pic
_FWDT(WDT_OFF);
//_FBORPOR(PBOR_OFF & BORV_27 & PWRT_16 & MCLR_EN);
_FBORPOR(PBOR_OFF & BORV_27 & PWRT_64 & MCLR_EN);
_FGS(CODE_PROT_OFF);
//-----------------------------------------------------------------------------
//Program Specific Constants
#define FCY 14740000					//Instruction cycle rate (Osc x PLL / 4) = 14.74 MIPS
#define MILLISEC FCY/14740			// 1 mSec delay constant

#define StartSendingK1on		PORTBbits.RB0				// input
#define StartSendingK1off		PORTBbits.RB1				// input
#define StartSendingK2on		PORTBbits.RB2				// input
#define StartSendingK2off		PORTBbits.RB3				// input
#define LED_KEY				LATBbits.LATB4			// Output
#define LED_RUN				LATBbits.LATB5			// Output
#define RF_TX					LATDbits.LATD0			// Output
#define RF_Alim				LATEbits.LATE8			// Output
#define LED_CN				LATDbits.LATD1			// Output

void setup_ports(void);
void DelayNmSec(unsigned int N);
void Delay(unsigned int N);
void InitUART();

struct {
			unsigned Running		: 	1;
			unsigned CheckRX		:		1;
			unsigned SendTX			:		1;
			unsigned SendData		:		1;
			unsigned RF_Sending	:		1;
			unsigned unused 		:		11;
		}	Flags;

//// RS232 -------------------------------------------
//unsigned char *TXPtr;
//unsigned char *RXPtr;
//void InitUART(void);
//void SendMsg(void);
//#define CR	0x0D
//#define LF 	0x0A
//#define BAUD 19200
//unsigned char InData[] = {"000000000"};
//unsigned char OutData[] = {"0000 0000 0000 0000 0000 0000 0000.00 0000.00 0000.00 0000.00  \r\n"};
//
//int SeqComm;		
//#define SeqCommMax 5000		// ttes les 0.5 s
//
//-----------------------------------------------------------------------------
//	Setup 
//-----------------------------------------------------------------------------
void setup_ports(void)
{
	ADPCFG=0xFFFF;	// all PORTB = Digital(1) : no ADC
	// Clear All Ports Prior to defining I/O
	PORTB=0;	//Initialize LED pin data to off state
	PORTC=0;
	PORTD=0;
	PORTE=0;	
	// Now set pin direction registers
	TRISB = 0xFFCF;		// RB0-3 inputs, RB0 StartSending PushB in, RB4 LED_KEY out,  RB5 LED_RUN out  1100|1111
	TRISC = 0xFFFF;		// U1ATX/RC13 in , U1ATX/RC14 out INUTILE de les configurer ainsi car directement on configure l'UART
	TRISD = 0xFFFC;		// RD1 RF24G_CE	out , RD0 RF24G_CS out
	TRISE = 0x00FF;		// RE8 RF_Alim out, le reste input	0|1111|1111
	TRISF = 0xFFFB;		// SDI1/RF2 in, SDO1/RF3 out : 1011 ou 1x11 car SDI1 doit se programmer via le TRISF
}

//-----------------------------------------------------------------------------
//	intitialise ChangeNotification CN7 et l interruption
//-----------------------------------------------------------------------------
void initChangeNotification(void)
{
	// CN2 enclench , on rising edge de Kon			PORTBbits.RB0
	// CN3 enclench , on rising edge de Koff			PORTBbits.RB1
//	CNEN1 = 0x000C;  //	RB0-RB1 donc CN2-3
	CNEN1 = 0x003C; //	RB0-RB3 donc CN2-5
	CNPU1= 0; CNPU2=0;	// No pull-up on input change
	IFS0bits.CNIF = 0;  // Clear IF bit
	IPC3bits.CNIP = 3;	// assigning Interrupt Priority to IPC Register
	IEC0bits.CNIE = 1;	// assiging Interrupt Enable
}
////---------------------------------------------------------------------
//// Below are the interrupt vectors for the serial receive and transmit
////---------------------------------------------------------------------	
//void __attribute__((interrupt, auto_psv)) _U1TXInterrupt(void)
//{
//	IFS0bits.U1TXIF = 0;	// clear interrupt flag
//}
////---------------------------------------------------------------------
//void __attribute__((interrupt, auto_psv)) _U1RXInterrupt(void)
//{
//	IFS0bits.U1RXIF = 0;	// clear interrupt flag
//	*RXPtr = U1RXREG;
//	if (*RXPtr == CR)
//		{Flags.CheckRX = 1;
//		RXPtr = &InData[0];}
//	else *RXPtr++;
//}
////------------------------------------------------------------------------
////	Transmission over serial
//void InitUART(void)
//{
//// Initialize the UART1 for BAUD = 19,200  
//	U1MODE = 0x8400;	// enable + alternate pins
////	U1MODE = 0x8000;	// enable + normal pins
//	U1STA = 0x0000;
//	U1BRG = ((FCY/16)/BAUD) - 1;	// set baud to 19200
//	IEC0bits.U1RXIE = 1;			// enable RX interrupt
//	RXPtr = &InData[0];		// point to first char in receive buffer
//	Flags.CheckRX = 0;			// clear rx and tx flags
//	Flags.SendTX = 0;
//	Flags.SendData = 0;		// clear flag 
//	SeqComm=SeqCommMax;
//	U1STAbits.UTXEN = 1;           // Initiate transmission
//}
////----------------------------------------------------
//// Convertit un "Word" hexa en "4 chars Hexadcimaux"
//// et les sort sur la table en memoire
////----------------------------------------------------
//inline void ConvHexa(int Var, int tablePos, unsigned char * table)
//{  
//int tmp;
//			tmp=Var & 0x000F;
//			if (tmp<=9)		tmp+=0x30;
//					else			tmp+=0x37;
//			table[tablePos+3]=tmp;
//
//			tmp=Var>>4 & 0x000F;
//			if (tmp<=9)		tmp+=0x30;
//					else			tmp+=0x37;
//			table[tablePos+2]=tmp;
//
//			tmp=Var>>8 & 0x000F;
//			if (tmp<=9)		tmp+=0x30;
//					else			tmp+=0x37;
//			table[tablePos+1]=tmp;
//
//			tmp=Var>>12 & 0x000F;
//			if (tmp<=9)		tmp+=0x30;
//					else			tmp+=0x37;
//			table[tablePos]=tmp;
//}
////----------------------------------------------------
//// Convertit un "12 bits" en "4 chars dcimaux. 2 chars" xxxx.xx
//// maximum : 1023.75 pour 12 bits
//// et les sort sur la table en memoire
////----------------------------------------------------
//inline void ConvDec(int Var, int tablePos, unsigned char * table)
//{  
//unsigned int k;
//unsigned char c;
//// Char
//	k = Var>>2;
//	c = k/1000;
//	if (c > 0)
//		k = k - c*1000;
//	table[tablePos]	 =(c + 0x30);
//	c = k/100;
//	if (c > 0)
//		k = k - c*100;
//	table[tablePos+1]=(c + 0x30);
//	c = k/10;
//	if (c > 0)
//		k = k - c*10;
//	table[tablePos+2]=(c + 0x30);
//	table[tablePos+3]=(char)(k + 0x30);
//	// apres la virgule :
//	switch (Var & 0x03)
//		{
//		case 0 :	table[tablePos+5]=0x30;		// xxxx.00
//							table[tablePos+6]=0x30;	break;
//		case 1 :	table[tablePos+5]=0x32;		// xxxx.25
//							table[tablePos+6]=0x35;	break;
//		case 2 :	table[tablePos+5]=0x35;		// xxxx.50
//							table[tablePos+6]=0x30;	break;
//		case 3 :	table[tablePos+5]=0x37;		// xxxx.75
//							table[tablePos+6]=0x35;	break;
//		}
//}
////-----------------------------------------------------------------------------
//void SendMsg(void)
//{
//while (*TXPtr)
//	{
//	while (U1STAbits.UTXBF);
//	U1TXREG = *TXPtr++;
//	}
//}
//------------------------------------------------------------------------
// SendData sends the debug information on the uart at 19200 baud
//void SendData()
//{
//int i,j;
//// Codage ASCII de la donne hexa
//	ConvHexa( TimeStamp, OffsetTimeStamp, OutData);				// TimeStamp en Hexa
//
//	// prpare OutData[] pour sortir en ascii sur UART en Hex
//	ConvHexa( TimeStamp_RX, OffsetTimeStamp_RX, OutData);	// TimeStamp_RX en Hexa
//	ConvHexa( Sensor0_T, OffsetSensor0_T, OutData);				// Sensor0_T en Hexa
//	ConvHexa( Sensor1_T, OffsetSensor1_T, OutData);				// Sensor1_T en Hexa
//	ConvHexa( Sensor2_T, OffsetSensor2_T, OutData);				// Sensor2_T en Hexa
//	ConvHexa( Sensor3_T, OffsetSensor3_T, OutData);				// Sensor3_T en Hexa
//	// prpare OutData[] pour sortir en ascii sur UART en Dcimal
//	ConvDec( Sensor0_Tr, OffsetSensor0_Tr, OutData);				// Sensor0_Tr en Decimal
//	ConvDec( Sensor1_Tr, OffsetSensor1_Tr, OutData);				// Sensor1_Tr en Decimal
//	ConvDec( Sensor2_Tr, OffsetSensor2_Tr, OutData);				// Sensor2_Tr en Decimal
//	ConvDec( Sensor3_Tr, OffsetSensor3_Tr, OutData);				// Sensor3_Tr en Decimal
//
//
//	TXPtr = &OutData[0];
//	SendMsg();
//}

//---------------------------------------------------------------------
// CN interrupt : Reception de donnes du RF24G, donnes prtes pour lecture
//---------------------------------------------------------------------
void __attribute__((interrupt, auto_psv)) _CNInterrupt( void )
{
	LED_RUN=1;
	IFS0bits.CNIF = 0;  // Clear IF bit
Delay(1000);	
	LED_RUN=0;	
}
////---------------------------------------------------------------------
//// Timer1 interrupt fait le calcul et l allumage des LED
//// ISR toutes les 100 us
////---------------------------------------------------------------------
//void __attribute__((interrupt, auto_psv)) _T1Interrupt( void )
//{
////	InterruptLED=1;	
//	IFS0bits.T1IF = 0;	
//
//	if (++tcnt>=tcntPRD)	
//						{// ici on est toutes les 1/4 secondes =0.25s
//						tcnt=0;
//						TimeStamp++;	// incrmente le registre de temps : 1 pas = 0.25s
//						// le read des donnes envoyes se fait sur un evnmt CN interrupt sur RF24G DR pin
//						// check appui sur PushButton (qui est gr par le dspic) 
//						if (StartCapture)	Flags.USBSendingAutorise = 1-Flags.USBSendingAutorise;	// Toggles le Flag d'autorisation d'envoi
//						}
//// communication dsPIC -> PC
//	if (!--SeqComm)		{
//										SeqComm=SeqCommMax;
//										Flags.SendData=1;
//										InfoLED=Flags.USBSendingAutorise;		// Allume si on est en Mode d'envoi de donnes sur USB/HID
//										}
//
////	InterruptLED=0;	
//}
//---------------------------------------------------------------------
void Delay(unsigned int N)
{
unsigned int j;
 	for(j=0;j < N; j++) __builtin_nop();
}
//-----------------------------------------------------------------------------
void SendX10RF_bit(unsigned int databit)
{
	// envoi un 1
	RF_TX=1;
	Delay(1600);		// 0.56 ms
	RF_TX=0;
	Delay(1600);		// 0.56 ms
	// si c'est un 0
	if (!databit)		Delay(3200);		// 0.56*2 = 1.12 ms si un zero
}
//-----------------------------------------------------------------------------
void SendX10byte(unsigned int data8)
{
unsigned int j, k;
	for (j=0; j<8; j++)
		{
		k=(data8 & 0x0080 )==0x0080;
		SendX10RF_bit(k);
		data8=data8 << 1;
		}
}
//-----------------------------------------------------------------------------
void SendX10Cmd(unsigned int data)
{
unsigned int Cmd8;
	RF_Alim=1;			// Alimente le transmetteur RE8=ON
	Delay(60000);		// >20 ms
// premebule
	RF_TX=1;
	Delay(26800);		// 8.96 ms
	RF_TX=0;
	Delay(13400);		// 4.5 ms
	Cmd8=data>>8;					// send les Hi
	SendX10byte(~Cmd8);
	SendX10byte(Cmd8);
	Cmd8=(data) & 0xFF;		// send les Low
	SendX10byte(~Cmd8);
	SendX10byte(Cmd8);
	SendX10RF_bit(1);			// 1  la fin ??
	Delay(3200);		// 1.12 ms
	RF_Alim=0;			// Eteint transmetteur RE8=OFF
}
//-----------------------------------------------------------------------------
void SendX10RF()
{
//	SendX10Cmd(0x6000);		// A1 ON
//	SendX10Cmd(0x6020);		// A1 OFF
	SendX10Cmd(0x6010);		// A2 ON
//	SendX10Cmd(0x6030);		// A2 OFF
//	SendX10Cmd(0x6090);		// A2 DIM+
//	SendX10Cmd(0x60B0);		// A2 DIM-

//http://idobartana.com/hakb/x10rf.htm
//10101111 11011101 01010101 11011111 10101101 01010101 <-A1 OFF cmd (as seen on RF, RIGHT bit first !)
//00000100 11111011 00000110 11111001    A1 OFF (as RIGHT bit first !)
// byte 1   byte 2   byte 3   byte 4
//bit  function
//0    always 0
//1    always 0
//2    always 0                               
//3    bit B1 of unit number
//4    bit B0 of unit number    
//5    1 for OFF command.                    DIM B2 ON/OFF B0 B1 <--(unit number -1= 0bB2B1B0  )
//6    bit B2 of unit number ------------------>|
//7    1 for DIM (if bit 5=1) or BRIGHT (if bit 5=0) command

//  l'envers aussi mais bytes inverses
//A- 00001 ON
//00032: 11111001 00000110 11111111 00000000	0x6000
//A- 00001 OFF
//00032: 11111001 00000110 11111011 00000100	0x6020
//A- 00002 ON
//00032: 11111001 00000110 11110111 00001000	0x6080
//A- 00002 OFF
//00032: 11111001 00000110 11110011 00001100	0x60C0
//A- 00002 DIM+
//00032: 11111001 00000110 11110111 00001000	0x6090
//http://camelspit.org/modified-x10-rf-transmitter/
}
//-----------------------------------------------------------------------------
//Main routine
int main(void)
{
unsigned int j;
	setup_ports();
	
// debug
	for (j=0; j<15000; j++)		__builtin_nop();		//  plein de dlais
//	InitUART();
//	TXPtr = &OutData[0];
//	SendMsg();

LED_RUN=0;

	initChangeNotification();

	Flags.RF_Sending=0;
	Flags.Running=1;

	Sleep();		// attends le premier ordre
	while(1)
		{	
		if (StartSendingK1on)	
			{
			LED_KEY=1;								// indique qu'on appuie sur la touche
			SendX10Cmd(0x6000);				// A1 ON
			while (StartSendingK1on)	DelayNmSec(10);		// wait 'till key is released
			DelayNmSec(500);					// pour pouvoir voir la LED
			LED_KEY=0;								// indique qu'on relachela touche
			Sleep();
			}
		
		if (StartSendingK1off)	
			{
			LED_KEY=1;								// indique qu'on appuie sur la touche
			SendX10Cmd(0x6020);				// A1 OFF
			while (StartSendingK1off)	DelayNmSec(10);		// wait 'till key is released
			DelayNmSec(500);					// pour pouvoir voir la LED
			LED_KEY=0;								// indique qu'on relachela touche
			Sleep();
			}
		
		if (StartSendingK2on)	
			{
			LED_KEY=1;								// indique qu'on appuie sur la touche
			SendX10Cmd(0x6010);				// A2 ON
			while (StartSendingK2on)	DelayNmSec(10);		// wait 'till key is released
			DelayNmSec(500);					// pour pouvoir voir la LED
			LED_KEY=0;								// indique qu'on relachela touche
			Sleep();
			}
		
		if (StartSendingK2off)	
			{
			LED_KEY=1;								// indique qu'on appuie sur la touche
			SendX10Cmd(0x6030);				// A2 OFF
			while (StartSendingK2off)	DelayNmSec(10);		// wait 'till key is released
			DelayNmSec(500);					// pour pouvoir voir la LED
			LED_KEY=0;								// indique qu'on relachela touche
			Sleep();
			}

//		if (Flags.SendData)
//				{
////				SendData();		// send present fs serially
//				Flags.SendData = 0;		// clear flag 
//				}
		}	// end of while (1)
}

//=============================================================================
//Error traps
//-----------------------------------------------------------------------------
//Oscillator Fail Error trap routine
void __attribute__((interrupt, auto_psv)) _OscillatorFail(void)
{	
	LED_RUN=0;	
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Address Error trap routine
void __attribute__((interrupt, auto_psv)) _AddressError(void)
{
	LED_RUN=0;	
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Stack Error trap routine
void __attribute__((interrupt, auto_psv)) _StackError(void)
{
	LED_RUN=0;	
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Math (Arithmetic) Error trap routine
void __attribute__((interrupt, auto_psv)) _MathError(void)
{
	LED_RUN=0;	
	while(1); //Wait forever
}
//---------------------------------------------------------------------
// This is a generic 1ms delay routine to give a 1mS to 65.5 Seconds delay
// For N = 1 the delay is 1 mS, for N = 65535 the delay is 65,535 mS. 
// Note that FCY is used in the computation.  Please make the necessary
// Changes(PLLx4 or PLLx8 etc) to compute the right FCY as in the define
// statement above.
//---------------------------------------------------------------------
void DelayNmSec(unsigned int N)
{
unsigned int j;
while(N--)
 	for(j=0;j < MILLISEC;j++);
}
//---------------------------------------------------------------------
